@@ -1,46 +1,57 @@
-#!/usr/bin/env python
 # emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
 # vi: set ft=python sts=4 ts=4 sw=4 et:
+#
+# Copyright 2021 The NiPreps Developers <nipreps@gmail.com>
+#
+# Licensed under the Apache License, Version 2.0 (the "License");
+# you may not use this file except in compliance with the License.
+# You may obtain a copy of the License at
+#
+#     http://www.apache.org/licenses/LICENSE-2.0
+#
+# Unless required by applicable law or agreed to in writing, software
+# distributed under the License is distributed on an "AS IS" BASIS,
+# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+# See the License for the specific language governing permissions and
+# limitations under the License.
+#
+# We support and encourage derived works from this project, please read
+# about our expectations at
+#
+#     https://www.nipreps.org/community/licensing/
+#
 """BIDS-Apps subject wrangler."""
-from builtins import range  # pylint: disable=W0622
-
-import os.path as op
 import glob
+import os.path as op
+from argparse import ArgumentParser, RawTextHelpFormatter
+from builtins import range  # pylint: disable=W0622
 from random import shuffle
-
-# from lockfile import LockFile
-
-from argparse import ArgumentParser
-from argparse import RawTextHelpFormatter
 from textwrap import dedent
 
-from .. import __version__
+from mriqc import __version__
+from mriqc.bin import messages
+
+COMMAND = "{exec} {bids_dir} {out_dir} participant --participant_label {labels} {work_dir} {arguments} {logfile}"  # noqa: E501
 
 
 def main():
-    """Entry point"""
+    """Entry point."""
     parser = ArgumentParser(
         formatter_class=RawTextHelpFormatter,
-        description=dedent(
-            """\
-BIDS-Apps participants wrangler tool
-------------------------------------
-
-This command arranges the participant labels in groups for computation, and checks that the \
-requested participants have the corresponding folder in the bids_dir.\
-"""
-        ),
+        description=dedent(messages.SUBJECT_WRANGLER_DESCRIPTION),
     )
 
     parser.add_argument(
-        "-v", "--version", action="version", version="mriqc v{}".format(__version__)
+        "-v",
+        "--version",
+        action="version",
+        version=f"mriqc v{__version__}",
     )
 
     parser.add_argument(
         "bids_dir",
         action="store",
-        help="The directory with the input dataset "
-        "formatted according to the BIDS standard.",
+        help="The directory with the input dataset formatted according to the BIDS standard.",
     )
     parser.add_argument(
         "output_dir",
@@ -67,35 +78,40 @@ requested participants have the corresponding folder in the bids_dir.\
         default=1,
         action="store",
         type=int,
-        help="parallelize participants in groups",
+        help="Parallelize participants in groups.",
     )
     parser.add_argument(
         "--no-randomize",
         default=False,
         action="store_true",
-        help="do not randomize participants list before grouping",
+        help="Do not randomize participants list before grouping.",
     )
     parser.add_argument(
-        "--log-groups", default=False, action="store_true", help="append logging output"
+        "--log-groups",
+        default=False,
+        action="store_true",
+        help="Append logging output.",
     )
     parser.add_argument(
         "--multiple-workdir",
         default=False,
         action="store_true",
-        help="split work directories by jobs",
+        help="Split work directories by jobs.",
     )
     parser.add_argument(
-        "--bids-app-name", default="mriqc", action="store", help="BIDS app to call"
+        "--bids-app-name",
+        default="mriqc",
+        action="store",
+        help="BIDS app to call.",
     )
-    parser.add_argument("--args", default="", action="store", help="append arguments")
+    parser.add_argument("--args", default="", action="store", help="Append arguments.")
 
     opts = parser.parse_args()
 
     # Build settings dict
     bids_dir = op.abspath(opts.bids_dir)
-    all_subjects = sorted(
-        [op.basename(subj)[4:] for subj in glob.glob(op.join(bids_dir, "sub-*"))]
-    )
+    subject_dirs = glob.glob(op.join(bids_dir, "sub-*"))
+    all_subjects = sorted([op.basename(subj)[4:] for subj in subject_dirs])
 
     subject_list = opts.participant_label
     if subject_list is None or not subject_list:
@@ -109,10 +125,10 @@ requested participants have the corresponding folder in the bids_dir.\
 
         if list(set(subject_list) - set(all_subjects)):
             non_exist = list(set(subject_list) - set(all_subjects))
-            raise RuntimeError(
-                "Participant label(s) not found in the "
-                "BIDS root directory: {}".format(" ".join(non_exist))
+            missing_label_error = messages.BIDS_LABEL_MISSING.format(
+                label=" ".join(non_exist)
             )
+            raise RuntimeError(missing_label_error)
 
     if not opts.no_randomize:
         shuffle(subject_list)
@@ -120,40 +136,26 @@ requested participants have the corresponding folder in the bids_dir.\
     gsize = opts.group_size
 
     if gsize < 0:
-        raise RuntimeError(
-            "group size should be at least 0 "
-            "(all participants assigned to same group"
-        )
+        raise RuntimeError(messages.BIDS_GROUP_SIZE)
     if gsize == 0:
         gsize = len(subject_list)
 
-    groups = [subject_list[i:i + gsize] for i in range(0, len(subject_list), gsize)]
+    j = i + gsize
+    groups = [subject_list[i:j] for i in range(0, len(subject_list), gsize)]
 
-    log_arg = "".format
-    if opts.log_groups:
-        log_arg = ">> log/mriqc-{:04d}.log".format
-
-    cmdline = (
-        "{exec} {bids_dir} {out_dir} participant --participant_label {labels}"
-        "{work_dir} {arguments} {logfile}"
-    ).format
+    log_arg = ">> log/mriqc-{:04d}.log" if opts.log_groups else ""
+    workdir_arg = " -w work/sjob-{:04d}" if opts.multiple_workdir else ""
     for i, part_group in enumerate(groups):
-        workdir = ""
-        if opts.multiple_workdir:
-            workdir = " -w work/sjob-{:04d}".format(i)
-        print(
-            cmdline(
-                **{
-                    "exec": opts.bids_app_name,
-                    "bids_dir": bids_dir,
-                    "out_dir": opts.output_dir,
-                    "labels": " ".join(part_group),
-                    "work_dir": workdir,
-                    "arguments": opts.args,
-                    "logfile": log_arg(i),
-                }
-            )
-        )
+        kwargs = {
+            "exec": opts.bids_app_name,
+            "bids_dir": bids_dir,
+            "out_dir": opts.output_dir,
+            "labels": " ".join(part_group),
+            "work_dir": workdir_arg.format(i),
+            "arguments": opts.args,
+            "logfile": log_arg.format(i),
+        }
+        print(COMMAND.format(**kwargs))
 
 
 if __name__ == "__main__":
