@@ -1,5 +1,25 @@
 # emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
 # vi: set ft=python sts=4 ts=4 sw=4 et:
+#
+# Copyright 2021 The NiPreps Developers <nipreps@gmail.com>
+#
+# Licensed under the Apache License, Version 2.0 (the "License");
+# you may not use this file except in compliance with the License.
+# You may obtain a copy of the License at
+#
+#     http://www.apache.org/licenses/LICENSE-2.0
+#
+# Unless required by applicable law or agreed to in writing, software
+# distributed under the License is distributed on an "AS IS" BASIS,
+# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+# See the License for the specific language governing permissions and
+# limitations under the License.
+#
+# We support and encourage derived works from this project, please read
+# about our expectations at
+#
+#     https://www.nipreps.org/community/licensing/
+#
 r"""
 A Python module to maintain unique, run-wide *MRIQC* settings.
 
@@ -38,7 +58,7 @@ graph is built across processes.
 .. code-block:: Python
 
     from mriqc import config
-    config_file = config.execution.work_dir / '.mriqc.toml'
+    config_file = mktemp(dir=config.execution.work_dir, prefix='.mriqc.', suffix='.toml')
     config.to_filename(config_file)
     # Call build_workflow(config_file, retval) in a subprocess
     with Manager() as mgr:
@@ -67,26 +87,33 @@ The :py:mod:`config` is responsible for other conveniency actions.
     :py:class:`~bids.layout.BIDSLayout`, etc.)
 
 """
-from ._warnings import logging
+import os
+import sys
+from pathlib import Path
+from time import strftime
+from uuid import uuid4
 
 try:
-    from multiprocessing import set_start_method
+    # This option is only available with Python 3.8
+    from importlib.metadata import version as get_version
+except ImportError:
+    from importlib_metadata import version as get_version
 
-    set_start_method("forkserver")
-except RuntimeError:
-    pass  # context has been already set
-finally:
-    # Defer all custom import for after initializing the forkserver and
-    # ignoring the most annoying warnings
-    import os
-    import sys
+# Ignore annoying warnings
+from mriqc._warnings import logging
 
-    from uuid import uuid4
-    from pathlib import Path
-    from time import strftime
-    from nipype import __version__ as _nipype_ver
-    from templateflow import __version__ as _tf_ver
-    from . import __version__
+__version__ = get_version("mriqc")
+_pre_exec_env = dict(os.environ)
+
+# Reduce numpy's vms by limiting OMP_NUM_THREADS
+_default_omp_threads = int(os.getenv("OMP_NUM_THREADS", os.cpu_count()))
+
+# Disable NiPype etelemetry always
+_disable_et = bool(
+    os.getenv("NO_ET") is not None or os.getenv("NIPYPE_NO_ET") is not None
+)
+os.environ["NIPYPE_NO_ET"] = "1"
+os.environ["NO_ET"] = "1"
 
 if not hasattr(sys, "_is_pytest_session"):
     sys._is_pytest_session = False  # Trick to avoid sklearn's FutureWarnings
@@ -124,14 +151,15 @@ if os.getenv("IS_DOCKER_8395080871"):
 
 _templateflow_home = Path(
     os.getenv(
-        "TEMPLATEFLOW_HOME", os.path.join(os.getenv("HOME"), ".cache", "templateflow")
+        "TEMPLATEFLOW_HOME",
+        os.path.join(os.getenv("HOME"), ".cache", "templateflow"),
     )
 )
 
 try:
     from psutil import virtual_memory
 
-    _free_mem_at_start = round(virtual_memory().free / 1024 ** 3, 1)
+    _free_mem_at_start = round(virtual_memory().free / 1024**3, 1)
 except Exception:
     _free_mem_at_start = None
 
@@ -158,6 +186,20 @@ try:
 except Exception:
     pass
 
+_memory_gb = None
+try:
+    if "linux" in sys.platform:
+        with open("/proc/meminfo", "r") as f_in:
+            _meminfo_lines = f_in.readlines()
+            _mem_total_line = [line for line in _meminfo_lines if "MemTotal" in line][0]
+            _mem_total = float(_mem_total_line.split()[1])
+            _memory_gb = _mem_total / (1024.0**2)
+    elif "darwin" in sys.platform:
+        _mem_str = os.popen("sysctl hw.memsize").read().strip().split(" ")[-1]
+        _memory_gb = float(_mem_str) / (1024.0**3)
+except Exception:
+    pass
+
 
 class _Config:
     """An abstract class forbidding instantiation."""
@@ -227,12 +269,16 @@ class environment(_Config):
     """Linux's kernel virtual memory overcommit policy."""
     overcommit_limit = _oc_limit
     """Linux's kernel virtual memory overcommit limits."""
-    nipype_version = _nipype_ver
+    nipype_version = get_version("nipype")
     """Nipype's current version."""
-    templateflow_version = _tf_ver
+    templateflow_version = get_version("templateflow")
     """The TemplateFlow client version installed."""
+    total_memory = _memory_gb
+    """Total memory available, in GB."""
     version = __version__
     """*MRIQC*'s version."""
+    _pre_mriqc = _pre_exec_env
+    """Environment variables before MRIQC's execution."""
 
 
 class nipype(_Config):
@@ -242,11 +288,13 @@ class nipype(_Config):
     """The file format for crashfiles, either text or pickle."""
     get_linked_libs = False
     """Run NiPype's tool to enlist linked libraries for every interface."""
+    local_hash_check = True
+    """Check if interface is cached locally before executing."""
     memory_gb = None
     """Estimation in GB of the RAM this workflow can allocate at any given time."""
     nprocs = os.cpu_count()
     """Number of processes (compute tasks) that can be run in parallel (multiprocessing only)."""
-    omp_nthreads = int(os.getenv('OMP_NUM_THREADS', os.cpu_count()))
+    omp_nthreads = _default_omp_threads
     """Number of CPUs a single process can access for multithreaded execution."""
     plugin = "MultiProc"
     """NiPype's execution plugin."""
@@ -255,6 +303,8 @@ class nipype(_Config):
         "raise_insufficient": False,
     }
     """Settings for NiPype's execution plugin."""
+    remove_node_directories = False
+    """Remove directories whose outputs have already been used up."""
     resource_monitor = False
     """Enable resource monitor."""
     stop_on_first_crash = True
@@ -268,7 +318,7 @@ class nipype(_Config):
             "plugin_args": cls.plugin_args,
         }
         if cls.plugin in ("MultiProc", "LegacyMultiProc"):
-            out["plugin_args"]["nprocs"] = int(cls.nprocs)
+            out["plugin_args"]["n_procs"] = int(cls.nprocs)
             if cls.memory_gb:
                 out["plugin_args"]["memory_gb"] = float(cls.memory_gb)
         return out
@@ -278,19 +328,6 @@ class nipype(_Config):
         """Set NiPype configurations."""
         from nipype import config as ncfg
 
-        # Configure resource_monitor
-        if cls.resource_monitor:
-            ncfg.update_config(
-                {
-                    "monitoring": {
-                        "enabled": cls.resource_monitor,
-                        "sample_frequency": "0.5",
-                        "summary_append": True,
-                    }
-                }
-            )
-            ncfg.enable_resource_monitor()
-
         # Nipype config (logs and execution)
         ncfg.update_config(
             {
@@ -311,8 +348,12 @@ class execution(_Config):
     """Use float number precision for ANTs computations."""
     bids_dir = None
     """An existing path to the dataset, which must be BIDS-compliant."""
+    bids_database_dir = None
+    """Path to the directory containing SQLite database indices for the input BIDS dataset."""
     bids_description_hash = None
     """Checksum (SHA256) of the ``dataset_description.json`` of the BIDS dataset."""
+    cwd = os.getcwd()
+    """Current working directory."""
     debug = False
     """Run in sloppy mode (meaning, suboptimal parameters that minimize run-time)."""
     dry_run = False
@@ -341,6 +382,8 @@ class execution(_Config):
     """Drop into PDB when exceptions are encountered."""
     reports_only = False
     """Only build the reports, based on the reportlets found in a cached working directory."""
+    resource_monitor = False
+    """Enable resource monitor."""
     run_id = None
     """Filter input dataset by run identifier."""
     run_uuid = "%s_%s" % (strftime("%Y%m%d-%H%M%S"), uuid4())
@@ -369,6 +412,7 @@ class execution(_Config):
     _paths = (
         "anat_derivatives",
         "bids_dir",
+        "bids_database_dir",
         "fs_license_file",
         "fs_subjects_dir",
         "layout",
@@ -383,31 +427,56 @@ class execution(_Config):
         """Create a new BIDS Layout accessible with :attr:`~execution.layout`."""
         if cls._layout is None:
             import re
+            from bids.layout.index import BIDSLayoutIndexer
             from bids.layout import BIDSLayout
 
-            work_dir = cls.work_dir / "bids.db"
-            work_dir.mkdir(exist_ok=True, parents=True)
-            cls._layout = BIDSLayout(
-                str(cls.bids_dir),
+            _db_path = cls.bids_database_dir or (
+                cls.work_dir / cls.run_uuid / "bids_db"
+            )
+            _db_path.mkdir(exist_ok=True, parents=True)
+
+            # Recommended after PyBIDS 12.1
+            _indexer = BIDSLayoutIndexer(
                 validate=False,
-                # database_path=str(work_dir),
                 ignore=(
                     "code",
                     "stimuli",
                     "sourcedata",
                     "models",
                     "derivatives",
+                    "scripts",
                     re.compile(r"^\."),
+                    # Exclude modalities and contrasts ignored by MRIQC (doesn't know how to QC)
+                    re.compile(
+                        r"sub-[a-zA-Z0-9]+(/ses-[a-zA-Z0-9]+)?/(dwi|fmap|perf)/"
+                    ),
+                    re.compile(
+                        r"sub-[a-zA-Z0-9]+(/ses-[a-zA-Z0-9]+)?/anat/.*_"
+                        r"(PDw|T2starw|FLAIR|inplaneT1|inplaneT2|PDT2|angio|T2star"
+                        r"|FLASH|PD|T1map|T2map|T2starmap|R1map|R2map|R2starmap|PDmap"
+                        r"|MTRmap|MTsat|UNIT1|T1rho|MWFmap|MTVmap|PDT2map|Chimap"
+                        r"|S0map|M0map|defacemask|MESE|MEGRE|VFA|IRT1|MP2RAGE|MPM|MTS|MTR)\."
+                    ),
+                    re.compile(
+                        r"sub-[a-zA-Z0-9]+(/ses-[a-zA-Z0-9]+)?/func/.*"
+                        r"_(cbv|sbref|phase|events|physio|stim)\."
+                    ),
                 ),
             )
+            cls._layout = BIDSLayout(
+                str(cls.bids_dir),
+                database_path=_db_path,
+                reset_database=cls.bids_database_dir is None,
+                indexer=_indexer,
+            )
+            cls.bids_database_dir = _db_path
+
         cls.layout = cls._layout
 
 
 # These variables are not necessary anymore
 del _exec_env
-del _nipype_ver
 del _templateflow_home
-del _tf_ver
 del _free_mem_at_start
 del _oc_limit
 del _oc_policy
@@ -438,10 +507,8 @@ class workflow(_Config):
     """Run ICA on the raw data and include the components in the individual reports."""
     inputs = None
     """List of files to be processed with MRIQC."""
-    start_idx = None
-    """Initial volume in functional timeseries that should be considered for preprocessing."""
-    stop_idx = None
-    """Final volume in functional timeseries that should be considered for preprocessing."""
+    species = "human"
+    """Subject species to choose most appropriate template"""
     template_id = "MNI152NLin2009cAsym"
     """TemplateFlow ID of template used for the anatomical processing."""
 
@@ -474,10 +541,9 @@ class loggers:
             * Logger configuration.
 
         """
-        from nipype import config as ncfg
-
         if not cls._init:
             from nipype import logging as nlogging
+            from nipype import config as ncfg
 
             cls.workflow = nlogging.getLogger("nipype.workflow")
             cls.interface = nlogging.getLogger("nipype.interface")
@@ -489,6 +555,15 @@ class loggers:
                     logging.Formatter(fmt=cls._fmt, datefmt=cls._datefmt)
                 )
                 cls.cli.addHandler(_handler)
+
+            ncfg.update_config(
+                {
+                    "logging": {
+                        "log_directory": str(execution.log_dir),
+                        "log_to_file": True,
+                    },
+                }
+            )
             cls._init = True
 
         cls.default.setLevel(execution.log_level)
@@ -496,9 +571,6 @@ class loggers:
         cls.interface.setLevel(execution.log_level)
         cls.workflow.setLevel(execution.log_level)
         cls.utils.setLevel(execution.log_level)
-        ncfg.update_config(
-            {"logging": {"log_directory": str(execution.log_dir), "log_to_file": True}, }
-        )
 
     @classmethod
     def getLogger(cls, name):
@@ -515,10 +587,9 @@ class loggers:
 
 def from_dict(settings):
     """Read settings from a flat dictionary."""
-    nipype.load(settings)
     execution.load(settings)
     workflow.load(settings)
-    loggers.init()
+    nipype.load(settings, init=False)
 
 
 def load(filename):
@@ -561,8 +632,22 @@ def dumps():
 def to_filename(filename):
     """Write settings to file."""
     filename = Path(filename)
+    filename.parent.mkdir(exist_ok=True, parents=True)
     filename.write_text(dumps())
 
 
-# Make sure loggers are started
-loggers.init()
+def _process_initializer(cwd, omp_nthreads):
+    """Initialize the environment of the child process."""
+    os.chdir(cwd)
+    os.environ["NIPYPE_NO_ET"] = "1"
+    os.environ["OMP_NUM_THREADS"] = f"{omp_nthreads}"
+
+
+def restore_env():
+    """Restore the original environment."""
+
+    for k in os.environ.keys():
+        del os.environ[k]
+
+    for k, v in environment._pre_mriqc.items():
+        os.environ[k] = v
